#include "opener_context_resource.h"
#include "opener_context_object_server.h"
#include "opener_resource.h"
#include "resource_server.h"

#include <string.h>



/* Support for providing access to files. */

OpenerContextResource::OpenerContextResource(OpenerResource *opener, Memory *memory)
: SimplePager(memory), _opener(opener)
{
}

int OpenerContextResource::expected_items()
{
    return OpenerContextObject_expected_items;
}

ipc_server_handler_type OpenerContextResource::handler()
{
    return (ipc_server_handler_type) handle_OpenerContextObject;
}



/* Data access methods. */

char *OpenerContextResource::get_path()
{
    char *buffer = _region->read();
    offset_t size = _region->size();

    /* Confine the path to the limit of the buffer. */

    if ((buffer != NULL) && (strnlen(buffer, size) < size))
        return buffer;
    else
        return NULL;
}



/* Opener context interface methods. */

long OpenerContextResource::open(flags_t flags, size_t *size, l4_cap_idx_t *file)
{
    char *path = get_path();

    if (path == NULL)
        return -L4_EINVAL;

    FilePager *pager = _opener->open(path, flags);

    /* Complete the initialisation and start a server in a new thread.
       If the thread does not start, the resource should be finalised. */

    ResourceServer server(pager);
    long err = server.start_thread();

    if (!err)
    {
        /* Return the file size. */
        /* Return the server capability to the caller. */

        *size = pager->get_data_size();
        *file = server.config()->server;
    }

    return err;
}

// vim: tabstop=4 expandtab shiftwidth=4
