#include "memory_incremental.h"
#include "pages.h"



Pages::Pages(Memory *memory, PageQueue *queue)
: _memory(memory), _queue(queue)
{
}

Pages::Pages(PageQueue *queue)
: _queue(queue)
{
    _memory = new MemoryIncremental();
}

Pages::~Pages()
{
    _queue->close(_memory);
}

/* Remove the first flexpage in the queue. If its owner exists, remove it from
   the owner (retiring the content). Return the flexpage for reuse. */

Flexpage *Pages::remove()
{
    PageOwner *owner;
    Flexpage *flexpage;

    _queue->pop(&owner, &flexpage);

    if (owner != NULL)
        owner->remove(flexpage);

    return flexpage;
}

/* Reserve 'flexpage' by removing it from this collection. */

bool Pages::reserve(PageOwner *owner, Flexpage *flexpage)
{
    return _queue->remove(owner, flexpage);
}

/* Obtain a new flexpage. Return the flexpage or None if no new flexpage could
   be created. */

Flexpage *Pages::flexpage()
{
    Region *region = _memory->region();

    if (region != NULL)
        return new Flexpage(region);
    else
        return NULL;
}

/* Queue an entry associating the given 'owner' and 'flexpage'. */

void Pages::queue(PageOwner *owner, Flexpage *flexpage)
{
    _queue->push(owner, flexpage);
}

/* Push an entry for 'flexpage' without owner association for immediate
   reuse. */

void Pages::release(Flexpage *flexpage)
{
    _queue->push_front(NULL, flexpage);
}

// vim: tabstop=4 expandtab shiftwidth=4
